#include "PauseMenu.hpp"
#include "../Controls.hpp"
#include "../ResourcePackManager.hpp"
#include "../UI/TextHelper.hpp"

#define BUILD_PC (BUILD_PLAT == BUILD_WINDOWS || BUILD_PLAT == BUILD_POSIX)

#if BUILD_PC
#define GLFW_INCLUDE_NONE
#include <GLFW/glfw3.h>

namespace GI {
extern GLFWwindow *window;
}
#endif

#if BUILD_PLAT == BUILD_3DS
#define SCREEN_W 400
#define SCREEN_H 240
#else
#define SCREEN_W 480
#define SCREEN_H 272
#endif
#define SCREEN_CENTER (SCREEN_W / 2)

namespace CrossCraft {

PauseMenu::PauseMenu() {
    gui_tex = ResourcePackManager::get().load_texture(
        "assets/minecraft/textures/gui/gui.png", SC_TEX_FILTER_NEAREST,
        SC_TEX_FILTER_NEAREST, false, true);

    font_texture = ResourcePackManager::get().load_texture(
        "assets/minecraft/textures/default.png", SC_TEX_FILTER_NEAREST,
        SC_TEX_FILTER_NEAREST, false, false);

    unsel_sprite = create_scopeptr<Graphics::G2D::Sprite>(
        gui_tex, Rendering::Rectangle{{SCREEN_CENTER - 100, 144}, {200, 20}},
        Rendering::Rectangle{{0, (256.0f - 86.0f) / 256.0f},
                             {200.0f / 256.0f, 20.0f / 256.0f}});

    unsel_sprite->set_layer(-4);

    sel_sprite = create_scopeptr<Graphics::G2D::Sprite>(
        gui_tex, Rendering::Rectangle{{SCREEN_CENTER - 100, 144}, {200, 20}},
        Rendering::Rectangle{{0, (256.0f - 106.0f) / 256.0f},
                             {200.0f / 256.0f, 20.0f / 256.0f}});
    sel_sprite->set_layer(-4);

    dis_sprite = create_scopeptr<Graphics::G2D::Sprite>(
        gui_tex, Rendering::Rectangle{{SCREEN_CENTER - 100, 144}, {200, 20}},
        Rendering::Rectangle{{0, (256.0f - 66.0f) / 256.0f},
                             {200.0f / 256.0f, 20.0f / 256.0f}});
    dis_sprite->set_layer(-4);

    fontRenderer = create_refptr<Graphics::G2D::FixedFontRenderer<256>>(
        font_texture, glm::vec2(16, 16));

    background_rectangle = create_scopeptr<Rendering::Primitive::Rectangle>(
        Rendering::Rectangle{{0, 0}, {SCREEN_W, SCREEN_H}},
        Rendering::Color{0, 0, 0, 96}, -3);
}
PauseMenu::~PauseMenu() {}

auto PauseMenu::enter() -> void {
#if BUILD_PC
    glfwSetInputMode(GI::window, GLFW_CURSOR, GLFW_CURSOR_NORMAL);
#endif

    pauseState = 0;
    selIdx = 0;
}
auto PauseMenu::exit() -> void {
#if BUILD_PC
    glfwSetInputMode(GI::window, GLFW_CURSOR, GLFW_CURSOR_DISABLED);
    Utilities::Input::set_cursor_center();
#endif

    selIdx = 0;
    pauseState = 0;
}

auto render_with_shadow(
    RefPtr<Graphics::G2D::FixedFontRenderer<256>> fontRenderer, std::string str,
    float x, float y) -> void {
    fontRenderer->add_text(str, {x - fontRenderer->calculate_size(str) / 2, y},
                           {255, 255, 255, 255}, -11);
    fontRenderer->add_text(
        str, {(x + 1) - fontRenderer->calculate_size(str) / 2, (y - 1)},
        {63, 63, 63, 255}, -10);
}

auto PauseMenu::draw() -> void {

#if BUILD_PC
    glClear(GL_DEPTH_BUFFER_BIT);
#endif

    background_rectangle->draw();
    fontRenderer->clear_tiles();

    switch (pauseState) {

    case 0: {
        Rendering::RenderContext::get().matrix_translate({0, 10, 0});

        if (selIdx == 0)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 1)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 2)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -48, 0});
        if (selIdx == 3)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        render_with_shadow(fontRenderer, "Game Menu", SCREEN_CENTER, 216);
        render_with_shadow(fontRenderer, "Back to Game", SCREEN_CENTER, 160);
        render_with_shadow(fontRenderer, "Options", SCREEN_CENTER, 136);
        render_with_shadow(fontRenderer, "Save Game", SCREEN_CENTER, 112);
        render_with_shadow(fontRenderer, "Quit Game", SCREEN_CENTER, 64);
        break;
    }

    case 1: {
        Rendering::RenderContext::get().matrix_translate({-108, 32, 0});

        if (selIdx == 0)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 1)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 2)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_clear();
        Rendering::RenderContext::get().matrix_translate({108, 32, 0});

        if (selIdx == 3)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 4)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 5)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_clear();
        Rendering::RenderContext::get().matrix_translate({0, -96 + 8, 0});
        if (selIdx == 6)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 7)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        render_with_shadow(fontRenderer, std::string("Option Menu:"),
                           SCREEN_CENTER, 216);

        render_with_shadow(fontRenderer,
                           std::string("Music: ") + (Option::get().music
                                                         ? std::string("ON")
                                                         : std::string("OFF")),
                           SCREEN_CENTER - 108, 182);

        auto dist = Option::get().renderDist;
        auto str = std::string("");
        switch (dist) {
        case 0: {
            str = "TINY";
            break;
        }
        case 1: {
            str = "SHORT";
            break;
        }
        case 2: {
            str = "NORMAL";
            break;
        }
        case 3: {
            str = "FAR";
            break;
        }
        }

        render_with_shadow(fontRenderer, std::string("Render Distance: ") + str,
                           SCREEN_CENTER - 108, 158);
        render_with_shadow(fontRenderer,
                           std::string("View Bobbing: ") +
                               (Option::get().bobbing ? std::string("ON")
                                                      : std::string("OFF")),
                           SCREEN_CENTER - 108, 134);

        render_with_shadow(fontRenderer,
                           std::string("Sound: ") + (Option::get().sound
                                                         ? std::string("ON")
                                                         : std::string("OFF")),
                           SCREEN_CENTER + 108, 182);
        render_with_shadow(
            fontRenderer,
            std::string("Show FPS: ") +
                (Option::get().fps ? std::string("ON") : std::string("OFF")),
            SCREEN_CENTER + 108, 158);
        render_with_shadow(fontRenderer,
                           std::string("Vsync: ") + (Option::get().vsync
                                                         ? std::string("ON")
                                                         : std::string("OFF")),
                           SCREEN_CENTER + 108, 134);

        render_with_shadow(fontRenderer, "Controls", SCREEN_CENTER, 62);
        render_with_shadow(fontRenderer, "Back", SCREEN_CENTER, 62 - 24);

        break;
    }

    case 2: {
#if !(BUILD_PLAT == BUILD_PSP || BUILD_PLAT == BUILD_VITA ||                   \
      BUILD_PLAT == BUILD_3DS)
        Rendering::RenderContext::get().matrix_translate({-108, 32, 0});

        if (selIdx == 0)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 1)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 2)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 3)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_clear();
        Rendering::RenderContext::get().matrix_translate({108, 32, 0});

        if (selIdx == 4)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 5)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 6)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 7)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_clear();
        Rendering::RenderContext::get().matrix_translate({0, -96 - 16, 0});
        if (selIdx == 8)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        render_with_shadow(fontRenderer, std::string("Controls:"), 240, 216);
        render_with_shadow(
            fontRenderer,
            std::string("Forward: ") +
                Controls::get().upperKeyName(Controls::get().keyForward),
            132, 182);
        render_with_shadow(fontRenderer,
                           std::string("Back: ") + Controls::get().upperKeyName(
                                                       Controls::get().keyBack),
                           132, 182 - 24);
        render_with_shadow(fontRenderer,
                           std::string("Jump: ") + Controls::get().upperKeyName(
                                                       Controls::get().keyJump),
                           132, 182 - 48);
        render_with_shadow(fontRenderer,
                           std::string("Chat: ") + Controls::get().upperKeyName(
                                                       Controls::get().keyChat),
                           132, 182 - 72);

        render_with_shadow(fontRenderer,
                           std::string("Left: ") + Controls::get().upperKeyName(
                                                       Controls::get().keyLeft),
                           348, 182);
        render_with_shadow(
            fontRenderer,
            std::string("Right: ") +
                Controls::get().upperKeyName(Controls::get().keyRight),
            348, 182 - 24);
        render_with_shadow(
            fontRenderer,
            std::string("Tab Menu: ") +
                Controls::get().upperKeyName(Controls::get().keyTab),
            348, 182 - 48);
        render_with_shadow(
            fontRenderer,
            std::string("Respawn: ") +
                Controls::get().upperKeyName(Controls::get().keyRespawn),
            348, 182 - 72);

        render_with_shadow(fontRenderer, "Back", 240, 62 - 24);
#else
        Rendering::RenderContext::get().matrix_translate({-108, 32, 0});
        if (selIdx == 0)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 1)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_clear();
        Rendering::RenderContext::get().matrix_translate({108, 32, 0});

        if (selIdx == 2)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 3)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_clear();
        Rendering::RenderContext::get().matrix_translate({0, -96 + 8, 0});
        if (selIdx == 4)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        Rendering::RenderContext::get().matrix_translate({0, -24, 0});
        if (selIdx == 5)
            sel_sprite->draw();
        else
            unsel_sprite->draw();

        render_with_shadow(fontRenderer, std::string("Controls:"),
                           SCREEN_CENTER, 216);
        render_with_shadow(
            fontRenderer,
            std::string("Break: ") +
                Controls::get().upperKeyName(Controls::get().buttonBreak),
            SCREEN_CENTER - 108, 182);
        render_with_shadow(
            fontRenderer,
            std::string("Place: ") +
                Controls::get().upperKeyName(Controls::get().buttonPlace),
            SCREEN_CENTER - 108, 182 - 24);
        render_with_shadow(
            fontRenderer,
            std::string("Block Menu: ") +
                Controls::get().upperKeyName(Controls::get().buttonMenu),
            SCREEN_CENTER + 108, 182);
        render_with_shadow(
            fontRenderer,
            std::string("Jump: ") +
                Controls::get().upperKeyName(Controls::get().buttonJump),
            SCREEN_CENTER + 108, 182 - 24);

#if PSP || BUILD_PLAT == BUILD_3DS
        render_with_shadow(fontRenderer,
                           std::string("Left Analog: ") +
                               (Controls::get().pspJoystickView
                                    ? std::string("View")
                                    : std::string("Move")),
                           SCREEN_CENTER, 62);
#else
        render_with_shadow(fontRenderer,
                           std::string("Left Analog: ") +
                               (Controls::get().vitaJoystickSwap
                                    ? std::string("View")
                                    : std::string("Move")),
                           SCREEN_CENTER, 62);
#endif
        render_with_shadow(fontRenderer, "Back", SCREEN_CENTER, 62 - 24);
#endif
        break;
    }
    }

    Rendering::RenderContext::get().matrix_clear();
    fontRenderer->generate_map();
    fontRenderer->draw();
}
auto PauseMenu::update() -> void {
#if BUILD_PC
    selIdx = -1;
    float cX =
        Utilities::Input::get_axis("Mouse", "X") * 480.0f / 2.0f + 240.0f;
    float cY =
        (1.0f - Utilities::Input::get_axis("Mouse", "Y")) * 272.0f / 2.0f;

    if (pauseState == 0) {
        if (cX >= 140.0f && cX <= 340.0f) {
            if (cY >= 154 && cY <= 174) {
                selIdx = 0;
            }
            if (cY >= 130 && cY <= 154) {
                selIdx = 1;
            }
            if (cY >= 106 && cY <= 126) {
                selIdx = 2;
            }
            if (cY >= 58 && cY <= 78) {
                selIdx = 3;
            }
        }
    } else if (pauseState == 1) {
        cY -= 22;
        if (cX >= 32 && cX <= 232) {
            if (cY >= 154 && cY <= 174) {
                selIdx = 0;
            }
            if (cY >= 130 && cY <= 154) {
                selIdx = 1;
            }
            if (cY >= 106 && cY <= 126) {
                selIdx = 2;
            }
        } else if (cX >= 248 && cX <= 448) {
            if (cY >= 154 && cY <= 174) {
                selIdx = 3;
            }
            if (cY >= 130 && cY <= 154) {
                selIdx = 4;
            }
            if (cY >= 106 && cY <= 126) {
                selIdx = 5;
            }
        }

        cY += 22;
        if (cX >= 140 && cX <= 340) {
            if (cY >= 56 && cY <= 76) {
                selIdx = 6;
            }
            if (cY >= 36 && cY <= 56) {
                selIdx = 7;
            }
        }
    } else if (pauseState == 2) {
        cY -= 22;
        if (cX >= 32 && cX <= 232) {
            if (cY >= 154 && cY <= 174) {
                selIdx = 0;
            }
            if (cY >= 130 && cY <= 154) {
                selIdx = 1;
            }
            if (cY >= 106 && cY <= 126) {
                selIdx = 2;
            }
            if (cY >= 82 && cY <= 102) {
                selIdx = 3;
            }
        } else if (cX >= 248 && cX <= 448) {
            if (cY >= 154 && cY <= 174) {
                selIdx = 4;
            }
            if (cY >= 130 && cY <= 154) {
                selIdx = 5;
            }
            if (cY >= 106 && cY <= 126) {
                selIdx = 6;
            }
            if (cY >= 82 && cY <= 102) {
                selIdx = 7;
            }
        }

        cY += 22;
        if (cX >= 140 && cX <= 340) {
            if (cY >= 36 && cY <= 56) {
                selIdx = 8;
            }
        }
    }
#endif
}

} // namespace CrossCraft
